#include "loader.h"
#include "iopmgr.h"

extern void *imgdrv_irx;
extern int size_imgdrv_irx;

extern void *usbd_irx;
extern int size_usbd_irx;

extern void *msd_irx;
extern int size_msd_irx;

static int first_IOP_reset = 1;

/*----------------------------------------------------------------------------------------*/

void list_modules(void)
{
	int c = 0;
	smod_mod_info_t info;
	u8 name[60];

	if (!smod_get_next_mod(0, &info))
		return;
	scr_printf("\n\t List of modules currently loaded in the system:\n\n");
	do {
		smem_read(info.name, name, sizeof name);
		if (!(c & 1)) scr_printf(" ");
		scr_printf("   \t%-21.21s  %2d  %3x%c", name, info.id, info.version, (++c & 1) ? ' ' : '\n');
	} while (smod_get_next_mod(&info, &info) != 0);
}


/*----------------------------------------------------------------------------------------*/
/* Reset IOP processor. This fonction hook reset iop if an update sequence is requested.  */
/*----------------------------------------------------------------------------------------*/
int New_Reset_Iop(const char *arg, int flag){
	void   *rom_iop;
	u8 *p;
	int     i, j, r, fd=0;
	ioprp_t ioprp_img;
	char    ioprp_path[0x50];
	int eeloadcnf_reset = 0;		
	
	GS_BGCOLOUR = 0xff00ff; 

	SifInitRpc(0);
	
	// check for reboot with IOPRP from cdrom
	char *ioprp_pattern = "rom0:UDNL cdrom";
	for (i=0; i<0x50; i++) {
		for (j=0; j<15; j++) {
			if (arg[i+j] != ioprp_pattern[j])
				break;
		}
		if (j == 15) {
			// get IOPRP img file path
			strcpy(ioprp_path, &arg[i+10]);
			break;
		}
	}

	// check for reboot with IOPRP from rom	
	char *eeloadcnf_pattern = "rom0:UDNL rom";
	for (i=0; i<0x50; i++) {
		for (j=0; j<13; j++) {
			if (arg[i+j] != eeloadcnf_pattern[j])
				break;
		}
		if (j == 13) {
			strcpy(ioprp_path, &arg[i+10]);
			eeloadcnf_reset = 1;
			break;
		}
	}	
		
	fioInit();
	fd = open(ioprp_path, O_RDONLY);
	if (fd < 0){
		GS_BGCOLOUR = 0x000080;
		while (1){;}
	}

	ioprp_img.size_in = lseek(fd, 0, SEEK_END);
	if (ioprp_img.size_in % 0x40)
		ioprp_img.size_in = (ioprp_img.size_in & 0xffffffc0) + 0x64;
	lseek(fd, 0, SEEK_SET);
		
	p = find_free_ram((void *)0x01700000, (void *)0x01f00000, ioprp_img.size_in + 65536);
	if (!p)
		ioprp_img.data_in  = (void *)0x01700000;
	else 
		ioprp_img.data_in  = (void *)p;

	p = find_free_ram((void *)0x017C0000, (void *)0x01f00000, ioprp_img.size_in + 65536);
	if (!p)
		ioprp_img.data_out  = (void *)0x017C0000;
	else 
		ioprp_img.data_out  = (void *)p;
	
	read(fd, ioprp_img.data_in , ioprp_img.size_in);
	close(fd);
	fioExit();

	if (eeloadcnf_reset) {
		r = Patch_EELOADCNF_Img(&ioprp_img);
	}
	else
		r = Patch_Img(&ioprp_img);

	if (r == 0){
		GS_BGCOLOUR = 0x00FF00;
		while (1){;}
	}

	SifExitRpc();
	SifExitIopHeap();
	LoadFileExit();
	
	/* Reseting IOP. */
	while (!Reset_Iop(NULL, 0)) {;}
	while (!Sync_Iop()){;}

	SifInitRpc(0);
	SifInitIopHeap();
	LoadFileInit();
	Sbv_Patch();

	rom_iop = SifAllocIopHeap(ioprp_img.size_out);
	
	if (rom_iop){
		CopyToIop(ioprp_img.data_out, ioprp_img.size_out, rom_iop);
		
		// get back imgdrv from kernel ram		
		DIntr();
		ee_kmode_enter();
		memcpy(g_buf, imgdrv_irx, size_imgdrv_irx);
		ee_kmode_exit();
		EIntr();

		*(u32*)(&g_buf[0x180]) = (u32)rom_iop;
		*(u32*)(&g_buf[0x184]) = ioprp_img.size_out;
		FlushCache(0);

		LoadMemModule(g_buf, size_imgdrv_irx, 0, NULL);
		LoadModuleAsync("rom0:UDNL", 5, "img0:");

		SifExitRpc();
		LoadFileExit();

		DIntr();
		ee_kmode_enter();
		Old_SifSetReg(SIF_REG_SMFLAG, 0x10000);
		Old_SifSetReg(SIF_REG_SMFLAG, 0x20000);
		Old_SifSetReg(0x80000002, 0);
		Old_SifSetReg(0x80000000, 0);
		ee_kmode_exit();
		EIntr();
	}else{
		GS_BGCOLOUR = 0xFF0000;
		while (1){;}
	}

	while (!Sync_Iop()) {;}

	SifExitIopHeap();
	SifInitRpc(0);
	SifInitIopHeap();
	LoadFileInit();
	Sbv_Patch();

	LoadIRXfromKernel(msd_irx, size_msd_irx, 0, NULL);
	
	SifExitRpc();
	SifExitIopHeap();
	LoadFileExit();

	// we have 4 SifSetReg calls to skip in ELF's SifResetIop, not when we use it ourselves
	if (!first_IOP_reset) {
		set_reg_hook = 4;
	}
	else
		first_IOP_reset = 0;

 return 1;
}

//--------------------------------------------------------------
void *find_free_ram(void *addr_start, void *addr_end, u32 len)
{
	u32 i, j;

	addr_start = (void *)(((u32)addr_start) & 0xffffffc0); // 64bytes align check
	
	u32 memsize = (u32)addr_end - (u32)addr_start;
	u8 *buf = (u8 *)addr_start;
	
	for (i = 0; i < memsize - len; ) {
		for (j = 0; j < len; j++) {
			if (buf[i + j] != 0)
				break;
		}
		if (j == len)
			return &buf[i];
		else
			i += (j + 0x40) & 0xffffffc0; // 64bytes align check
	}
	
	return NULL;
}


/*----------------------------------------------------------------------------------------*/
/* Reset IOP processor. This fonction replace SifIopReset from Ps2Sdk                     */
/*----------------------------------------------------------------------------------------*/
int Reset_Iop(const char *arg, int flag){
	SifCmdResetData reset_pkt;
	struct t_SifDmaTransfer dmat;

	SifExitRpc();

	SifStopDma();

	memset(&reset_pkt, 0, sizeof(SifCmdResetData));

	reset_pkt.chdr.psize = sizeof(SifCmdResetData);
	reset_pkt.chdr.fcode = 0x80000003;

	reset_pkt.flag = flag;

	if (arg != NULL){
		strncpy(reset_pkt.arg, arg, 0x50);
		reset_pkt.arg[0x50] = '\0';
		reset_pkt.size = strlen(reset_pkt.arg) + 1;
	}

	dmat.src  = &reset_pkt;
	dmat.dest = (void *)SifGetReg(0x80000000); /* SIF_REG_SUBADDR */
	dmat.size = sizeof reset_pkt;
	dmat.attr = 0x40 | SIF_DMA_INT_O;
	SifWriteBackDCache(&reset_pkt, sizeof reset_pkt);

	DIntr();
	ee_kmode_enter();
	if (!Old_SifSetDma(&dmat, 1)){
		ee_kmode_exit();
		EIntr();
		return 0;
	}

	Old_SifSetReg(SIF_REG_SMFLAG, 0x10000);
	Old_SifSetReg(SIF_REG_SMFLAG, 0x20000);
	Old_SifSetReg(0x80000002, 0);
	Old_SifSetReg(0x80000000, 0);
	ee_kmode_exit();
	EIntr();

	return 1;
}


/*----------------------------------------------------------------------------------------*/
/* Synchronize IOP processor. This fonction replace SifIopReset from Ps2Sdk                     */
/*----------------------------------------------------------------------------------------*/
int Sync_Iop(void){
	if (SifGetReg(SIF_REG_SMFLAG) & 0x40000){
		return 1;
	}
	return 0;
}
